﻿------------------------------------------------------------
-- OptionFrame.lua
--
-- Abin
-- 2010/10/16
------------------------------------------------------------

local CreateFrame = CreateFrame
local pairs = pairs
local ipairs = ipairs
local type = type
local tostring = tostring
local tonumber = tonumber
local GetSpellInfo = GetSpellInfo
local strtrim = strtrim
local ColorPickerFrame = ColorPickerFrame

local L = CompactRaid:GetLocale("CornerIndicators")
local module = CompactRaid:GetModule("CornerIndicators")
if not module then return end

local templates = CompactRaid.optionTemplates

local page = module.optionPage

local panel = UICreateTabFrame(page:GetName().."Panel", page)
page.tabFrame = panel
page:AnchorToTopLeft(panel, 0, -24)
panel:SetWidth(480)
panel:SetHeight(400)

panel:AddTab(L["top left"])
panel:AddTab(L["top right"])
panel:AddTab(L["bottom left"])
panel:AddTab(L["bottom right"])

local scaleoffset = templates:CreateScaleOffsetGroup(page)
scaleoffset:SetPoint("TOPLEFT", panel, "TOPLEFT", 20, -20)

function scaleoffset:OnScaleApply(scale)
	module.chardb[module.activeKey].scale = scale
	module:UpdateAllIndicators(module.activeKey, nil, nil, 1)
end

function scaleoffset:OnScaleCancel()
	return module.chardb[module.activeKey].scale
end

function scaleoffset:OnOffsetApply(xoffset, yoffset)
	module.chardb[module.activeKey].xoffset, module.chardb[module.activeKey].yoffset = xoffset, yoffset
	module:UpdateAllIndicators(module.activeKey, nil, nil, nil, 1)
end

function scaleoffset:OnOffsetCancel()
	return module.chardb[module.activeKey].xoffset, module.chardb[module.activeKey].yoffset
end

local styleGroup = page:CreateSingleSelectionGroup(L["indicator style"], 1)
styleGroup:SetPoint("TOPLEFT", scaleoffset, "BOTTOMLEFT", 0, -20)
styleGroup:AddButton(L["icon"], 0):ClearAllPoints()
styleGroup[1]:SetPoint("LEFT", styleGroup, "LEFT", 48, 0)
styleGroup:AddButton(L["color block"], 1)
styleGroup:AddButton(L["numerical"], 2)
styleGroup:AddButton(HIDE, 3)

function styleGroup:OnCheckInit(value)
	return value == module.activedb.style
end

function styleGroup:OnSelectionChanged(value)
	module.activedb.style = value
	page:UpdateOptionStats()
	module:UpdateAllIndicators(module.activeKey, 1)
end

local function CreateStageOption(text, key)
	local colorSwatch = templates:CreateColorSwatch(nil, page)
	colorSwatch.key = key

	local label = page:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft")
	colorSwatch.label = label
	label:SetPoint("LEFT", colorSwatch, "RIGHT")
	label:SetText(" - "..text)

	if key then
		local combo = page:CreateComboBox()
		colorSwatch.combo = combo
		combo.key = key
		combo:SetPoint("LEFT", label, "RIGHT", 7, 0)
		combo:SetWidth(100)

		if key == "1" then
			combo:AddLine("75%", 75)
			combo:AddLine("50%", 50)
			combo:AddLine("25%", 25)
			combo:AddLine(NONE)
		else
			combo:AddLine(format(INT_SPELL_DURATION_SEC, 10), 10)
			combo:AddLine(format(INT_SPELL_DURATION_SEC, 7), 7)
			combo:AddLine(format(INT_SPELL_DURATION_SEC, 5), 5)
			combo:AddLine(format(INT_SPELL_DURATION_SEC, 3), 3)
			combo:AddLine(NONE)
		end

		combo.OnComboChanged = function(self, value)
			module.activedb[self.key].threshold = value
		end
	end

	colorSwatch.OnColorChange = function(self, r, g, b)
		local stage = module.activedb[self.key] or module.activedb
		stage.r, stage.g, stage.b = r, g, b
	end

	colorSwatch.OnEnable = function(self)
		self.label:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		if self.combo then
			self.combo:Enable()
		end
	end

	colorSwatch.OnDisable = function(self)
		self.label:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
		if self.combo then
			self.combo:Disable()
		end
	end

	return colorSwatch
end

local auraEdit = page:CreateEditBox(L["aura name"], 1)
auraEdit:SetWidth(170)
auraEdit.text:ClearAllPoints()
auraEdit.text:SetPoint("TOPLEFT", styleGroup, "BOTTOMLEFT", 0, -24)
auraEdit:SetPoint("LEFT", auraEdit.text, "LEFT", 50, 0)
templates:SetEditBoxAutoPrompt(auraEdit)

function auraEdit:OnTextCommit(text)
	text = strtrim(text)
	if text == "" then
		text = nil
	end

	local spellId = tonumber(text)
	if spellId then
		text = GetSpellInfo(spellId)
		self:SetText(text or "")
	end

	module.activedb.aura = text
	page:UpdateSimilarPrompt()
	module:UpdateAllIndicators(module.activeKey, nil, 1)
end

function auraEdit:OnTextCancel()
	return module.activedb.aura or ""
end

auraEdit:HookScript("OnEditFocusGained", function(self)
	CloseDropDownMenus()
end)

local similarButton = page:CreateSubControl("Button")
similarButton.icon = similarButton:CreateTexture(nil, "BORDER")
similarButton.icon:SetAllPoints(similarButton)
similarButton.icon:SetTexture("Interface\\Icons\\Ability_Rogue_ShadowStrikes")
similarButton.count = similarButton:CreateFontString(nil, "ARTWORK", "TextStatusBarText")
similarButton.count:SetPoint("BOTTOMRIGHT", -1, 1)
similarButton:SetSize(20, 20)
similarButton:SetPoint("LEFT", auraEdit, "RIGHT", 4, 0)

function page:UpdateSimilarPrompt()
	local aura = module.activedb.aura
	local list = module:GetGroupBuffs(module:GetBuffGroup(aura))
	if list then
		module.activedb.checkSimilar = 1
		similarButton.tooltipTitle = aura
		local similar, text
		local count = 0
		for similar in pairs(list) do
			if similar ~= aura then
				count = count + 1
				if text then
					text = text..", "..similar
				else
					text = similar
				end
			end
		end

		similarButton.tooltipText = L["similars/conflicts"].."|cffffffff"..(text or "").."|r"
		similarButton.icon:SetDesaturated(false)
		similarButton.count:SetText(count)
	else
		module.activedb.checkSimilar, similarButton.tooltipTitle, similarButton.tooltipText = nil
		similarButton.icon:SetDesaturated(true)
		similarButton.count:SetText()
	end

	if GameTooltip:IsOwned(similarButton) then
		similarButton:Hide()
		similarButton:Show()
	end
end

local spellButton = page:CreateSubControl("Button")
spellButton:SetWidth("28")
spellButton:SetHeight("58")
spellButton:SetScale(0.7)
spellButton:SetPoint("LEFT", similarButton, "RIGHT", 4, 11)
LoadMicroButtonTextures(spellButton, "Spellbook")
spellButton:SetHitRectInsets(0, 0, 22, 0)

local BUILTIN_SPELLS = module.DEFAULT_SPELLS[select(2, UnitClass("player"))]
if BUILTIN_SPELLS then
	local spellList = {}
	local spellid
	for _, spellid in ipairs(BUILTIN_SPELLS) do
		local name, _, icon = GetSpellInfo(spellid)
		if name then
			spellList[name] = icon
		end
	end

	local spellMenu = CreateFrame("Button", spellButton:GetName().."SpellMenu", spellButton, "UIDropDownMenuTemplate")
	spellMenu.point = "TOPRIGHT"
	spellMenu.relativeTo = spellButton
	spellMenu.relativePoint = "TOPLEFT"
	spellMenu.xOffset = 0
	spellMenu.yOffset = -14

	local function OnMenuClick(self, spell)
		if spell then
			auraEdit:SetText(spell)
			auraEdit:OnTextCommit(spell)
			auraEdit:ClearFocus()
		end
	end

	UIDropDownMenu_Initialize(spellMenu, function()
		local text = strtrim(auraEdit:GetText())
		local name, icon
		for name, icon in pairs(spellList) do
			local sel = name == text
			UIDropDownMenu_AddButton({ text = name, icon = icon, arg1 = name, func = OnMenuClick, checked = sel })
		end
	end, "MENU")

	spellButton:SetScript("OnClick", function()
		auraEdit:ClearFocus()
		ToggleDropDownMenu(nil, nil, spellMenu)
	end)
else
	spellButton:Disable()
end

function page:UpdateOptionStats()
	local i
	for i = 0, 2 do
		local colorSwatch = self.stages[tostring(i)]
		if module.activedb.style == 0 or module.activedb.showlacks then
			colorSwatch:Disable()
		else
			colorSwatch:Enable()
		end
	end
end

local spellGroup = page:CreateMultiSelectionGroup(CALENDAR_FILTERS, 1)
spellGroup:SetPoint("TOPLEFT", auraEdit.text, "BOTTOMLEFT", 0, -20)

spellGroup:AddButton(L["self cast"], "selfcast")
spellGroup:AddButton(L["show lacks"], "showlacks"):ClearAllPoints()
spellGroup:AddButton(L["ignore outranged"], "ignoreOutRanged"):ClearAllPoints()
spellGroup:AddButton(L["ignore vehicles"], "ignoreVehicle"):ClearAllPoints()
spellGroup:AddButton(L["ignore physical classes"], "ignorePhysical"):ClearAllPoints()
spellGroup:AddButton(L["ignore magical classes"], "ignoreMagical"):ClearAllPoints()
page.showLacksCheck = spellGroup[2]

-- [aura name]   [book]
-- [1] selfcast		[2] show misses
-- [3] ignore outranged	[4] ignore vehicle
-- [5] ignore physical	[6] ignore magical

spellGroup[2]:SetPoint("LEFT", spellGroup[1], "LEFT", 210, 0)
spellGroup[3]:SetPoint("TOPLEFT", spellGroup[1], "BOTTOMLEFT")
spellGroup[4]:SetPoint("TOPLEFT", spellGroup[2], "BOTTOMLEFT")
spellGroup[5]:SetPoint("TOPLEFT", spellGroup[3], "BOTTOMLEFT")
spellGroup[6]:SetPoint("TOPLEFT", spellGroup[4], "BOTTOMLEFT")

function spellGroup:OnCheckInit(value)
	return module.activedb[value]
end

function spellGroup:OnCheckChanged(value, checked)
	module.activedb[value] = checked
	module:UpdateAllIndicators(module.activeKey, nil, 1)
	if value == "showlacks" then
		page:UpdateOptionStats()
	end
end

local colorLabel = page:CreateFontString(nil, "ARTWORK", "GameFontNormal")
colorLabel:SetPoint("TOPLEFT", spellGroup[5], "BOTTOMLEFT", 0, -20)
colorLabel:SetText(L["stage colors"])

page.stages = {}
page.stages["0"] = CreateStageOption(L["normal stage"])
page.stages["0"]:SetPoint("TOPLEFT", colorLabel, "BOTTOMLEFT", 8, -10)

page.stages["1"] = CreateStageOption(L["remaining time"], "1")
page.stages["1"]:SetPoint("TOPLEFT", page.stages["0"], "BOTTOMLEFT", 0, -8)

page.stages["2"] = CreateStageOption(L["remaining time"], "2")
page.stages["2"]:SetPoint("TOPLEFT", page.stages["1"], "BOTTOMLEFT", 0, -8)

local function LoadStageData(stage)
	local db = stage and module.activedb[stage] or module.activedb
	local colorSwatch = page.stages[stage or "0"]
	colorSwatch:SetColor(db.r, db.g, db.b)
	if stage then
		colorSwatch.combo:SetSelection(db.threshold, 1)
	end
end

function panel:OnTabSelected(id, value)
	scaleoffset:ClearFocus()
	auraEdit:ClearFocus()
	ColorPickerFrame:Hide()
	CloseDropDownMenus()
	module.activeKey = module.INDICATOR_KEYS[id]
	module.activedb = module.talentdb[module.activeKey]
	if not module.activedb then
		return -- Should never happen
	end

	scaleoffset:SetValue("scale", module.chardb[module.activeKey].scale)
	scaleoffset:SetValue("offset", module.chardb[module.activeKey].xoffset, module.chardb[module.activeKey].yoffset)
	styleGroup:SetSelection(module.activedb.style, 1)
	auraEdit:SetText(module.activedb.aura or "")
	spellGroup:SetChecked("selfcast", module.activedb.selfcast, 1)
	spellGroup:SetChecked("showlacks", module.activedb.showlacks, 1)
	spellGroup:SetChecked("ignoreOutRanged", module.activedb.ignoreOutRanged, 1)
	spellGroup:SetChecked("ignoreVehicle", module.activedb.ignoreVehicle, 1)
	spellGroup:SetChecked("ignorePhysical", module.activedb.ignorePhysical, 1)
	spellGroup:SetChecked("ignoreMagical", module.activedb.ignoreMagical, 1)
	page:UpdateSimilarPrompt()
	LoadStageData()
	LoadStageData("1")
	LoadStageData("2")
	page:UpdateOptionStats()
end

local defaultButton = page:CreatePressButton(DEFAULT)
defaultButton:SetPoint("TOPRIGHT", panel, "BOTTOMRIGHT", 0, -6)
defaultButton:SetWidth(96)
defaultButton:SetScript("OnClick", function(self) CompactRaid:RestoreModuleDefaults(module) end)